# -*- coding: utf-8 -*-

import sys
import os

import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk

from pluginMgr import PluginMgr
from infoWindow import InfoWindow


# GTKスレッド初期化
gtk.gdk.threads_init()

# 主要ディレクトリの絶対パス文字列を取得
STR_PLUGGER_DIR = os.path.abspath(os.path.dirname(sys.argv[0]))
if sys.platform == 'win32':
    # Windows環境でpy2exeを使用した場合でも正しくパスが取得できるように
    import imp
    if (hasattr(sys, 'frozen') or
        hasattr(sys, 'importers') or
        imp.is_frozen('__main__')):
        STR_PLUGGER_DIR = os.path.abs.path(os.path.split(sys.excecutable))

STR_PATH_PLUGINS = os.path.normpath(os.path.join(STR_PLUGGER_DIR, 'plugins'))
STR_PATH_CONFIGS = os.path.normpath(os.path.join(STR_PLUGGER_DIR, 'configs'))

#
#
class PlugWindow:
    '''
    pyPluggerのメインウインドウ
    '''
    def __init__(self, pluginMgr=None):
        '''
        初期化ウインドウ
        '''
        self.pluginMgr = pluginMgr
        self.pluginContainer = None
        self.authorLabel = None
        self.versionLabel = None
        self.descLabel = None
        
        # ウインドウの生成
        self.wind = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.wind.set_border_width(6)
        self.wind.set_position(gtk.WIN_POS_CENTER)
        self.wind.set_title('pyPlugger')        
        self.wind.connect('delete-event', self.on_delete_event)

        # ウインドウ内容を構築
        widgets = self.get_widgets()
        if widgets:
            self.wind.add(widgets)

        # 最初に表示するプラグインのウィジェットを表示
        if self.pluginCombo:
            self.on_change_plugin_combo(self.pluginCombo)
        
        # ウインドウの内容を表示
        self.wind.show_all()

    def get_widgets(self):
        '''
        ウインドウに表示するウィジェットを構築する。
        '''
        vbox = gtk.VBox()

        hbox = gtk.HBox()
        vbox.pack_start(hbox, False, False, 5)

        img = gtk.image_new_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
        bt = gtk.Button('')
        bt.set_image(img)
        bt.connect('clicked', self.on_click_info)
        hbox.pack_start(bt, False, False, 4)
        
        label = gtk.Label('Choose Plugin')
        label.set_size_request(100, -1)
        hbox.pack_start(label)
        
        self.pluginCombo = gtk.combo_box_new_text()
        self.pluginCombo.connect('changed', self.on_change_plugin_combo)
        hbox.pack_start(self.pluginCombo)
        self.update_plugin_combobox()

        self.pluginContainer = gtk.Frame()
        vbox.pack_start(self.pluginContainer, False, False, 4)

        expander = gtk.Expander('Plugin info')
        vbox.pack_start(expander, False, False, 4)

        expVBox = gtk.VBox(False)
        expander.add(expVBox)

        label = gtk.Label('Author:')
        label.set_alignment(0.0, 0.5)
        expVBox.pack_start(label, False, False, 2)

        self.authorLabel = gtk.Label('----')
        self.authorLabel.set_alignment(0.1, 0.5)
        expVBox.pack_start(self.authorLabel, False, False, 2)
        
        label = gtk.Label('Version:')
        label.set_alignment(0.0, 0.5)
        expVBox.pack_start(label, False, False, 2)
        self.versionLabel = gtk.Label('----')
        self.versionLabel.set_alignment(0.1, 0.5)
        expVBox.pack_start(self.versionLabel, False, False, 2)

        label = gtk.Label('Description:')
        label.set_alignment(0.0, 0.5)
        expVBox.pack_start(label, False, False, 2)
        self.descLabel = gtk.Label('----')
        self.descLabel.set_alignment(0.2, 0.5)
        expVBox.pack_start(self.descLabel, False, False, 2)

        return vbox

    def update_plugin_combobox(self):
        '''
        プラグインコンボボックスの項目名を更新する。
        '''
        if self.pluginMgr == None:
            return
        if self.pluginCombo == None:
            return

        # コンボボックスにプラグインから取得した名称を
        # 追加する。
        for name in self.pluginMgr.get_plugin_names():
            self.pluginCombo.append_text(name)
        self.pluginCombo.set_active(0)

    def on_click_info(self, widget, event=None):
        '''
        情報ダイアログの表示
        '''
        infoWind = InfoWindow(self.wind)

    def on_change_plugin_combo(self, combo, data=None):
        '''
        コンボボックスの選択状態が変更された場合の処理
        '''
        if (combo == None or
            self.pluginMgr == None or
            self.pluginContainer == None):
            return

        # コンボボックスからプラグインモジュール名を取得
        pluginName = combo.get_active_text()
        
        # プラグインのGUI作成関数を取得する。
        # 存在しなければ処理を中断
        func_get_widgets = self.pluginMgr.get_plugin_func(pluginName, 'get_widgets')
        if func_get_widgets == None:
            return
        
        # 現在表示されているプラグインのGUIがあれば削除する。
        child = self.pluginContainer.get_child()
        if child:
            self.pluginContainer.remove(child)

        # プラグインにGUIを作成させ、本体に貼り付ける。        
        widget = func_get_widgets()
        if widget:
            self.pluginContainer.add(widget)
            self.pluginContainer.show_all()

        # プラグインのサイズ調整
        # 最小サイズに設定することで、プラグインごとのデフォルトサイズに設定
        self.wind.resize(1, 1)
        
        # プラグインについての情報(作者、バージョン、説明など)
        # を更新する。
        infoList = [('get_author', self.authorLabel),
                    ('get_version', self.versionLabel),
                    ('get_description', self.descLabel)]
        for funcName, field in infoList:
            func_get_info = self.pluginMgr.get_plugin_func(pluginName, funcName)
            if func_get_info == None:
                continue
            field.set_text(func_get_info())
                        
    def on_delete_event(self, widget, event=None):
        '''
        ウインドウが閉じられた時に実行すべき処理
        '''
        if self.pluginMgr:
            self.pluginMgr.save_all_configs(STR_PATH_CONFIGS,
                                            'get_settings')

        print '-- quit --'
        gtk.main_quit()
        return False


#
#
if __name__ == '__main__':
    '''
    このスクリプトが単体で実行された場合のみツールを起動する。
    '''
    # プラグインプロトコルに適合する関数の条件を準備
    funcs = [('get_name', True),
             ('get_version', True),
             ('get_widgets', True),
             ('get_author', False),
             ('get_description', False),
             ('get_settings', False),
             ('set_settings', False)]

    # プラグインマネージャの生成
    pluginMgr = PluginMgr(STR_PATH_PLUGINS, funcs)
    pluginMgr.load_all_configs(STR_PATH_CONFIGS, 'set_settings')
    
    # ウインドウを生成
    wind = PlugWindow(pluginMgr)
    
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()
    
