# -*- coding: utf-8 -*-

import os
import sys
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import cairo

#
#
class ShapedWindow(gtk.Window):
    '''
    PNG画像によって形状をマスクされたウインドウ。
    生成時に指定されたPNG画像の形状のウインドウとなる。
    '''
    def __init__(self, parent=None, imageName=None):
        '''
        初期化。
        imageNameで渡された画像を使って形状をマスクしたウインドウを生成する。
        isModalがTrueの場合、モーダルなウインドウとなる。
        '''
        gtk.Window.__init__(self)

        # ウインドウを生成
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_decorated(False)

        if imageName == None:
            return
        if os.path.isfile(imageName) == False:
            return
        
        # ウインドウの背景イメージを読み込む
        img = gtk.Image()
        img.set_from_file(imageName)

        self.box = gtk.EventBox()
        self.box.add(img)
        self.add(self.box)
        self.set_resizable(False)

        buff = img.get_pixbuf()
        self.bitmap = gtk.gdk.Pixmap(None,
                                     buff.get_width(),
                                     buff.get_height(),
                                     1)
        ca = self.bitmap.cairo_create()

        ca.set_source_rgb(0, 0, 0)
        ca.set_operator(cairo.OPERATOR_DEST_OUT)
        ca.paint()

        img = cairo.ImageSurface.create_from_png(imageName)
        ca.set_source_surface(img)
        ca.set_operator(cairo.OPERATOR_OVER)
        ca.paint()
        
        # ウインドウの形状をマスクする。
        self.shape_combine_mask(self.bitmap, 0, 0)

#
#
if __name__ == '__main__':
    wind = ShapedWindow(imageName='logo.png')
    wind.show_all()

    gtk.main()
